///////////////////////
/* VistaElevator.cpp - v.2.0

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Copyright (C) 2007.  WinAbility Software Corporation. All rights reserved.

Author: Andrei Belogortseff [ http://www.tweak-uac.com ]

TERMS OF USE: You are free to use this file in any way you like, 
for both the commercial and non-commercial purposes, royalty-free,
AS LONG AS you agree with the warranty disclaimer above, 
EXCEPT that you may not remove or modify this or any of the 
preceeding paragraphs. If you make any changes, please document 
them in the MODIFICATIONS section below. If the changes are of general 
interest, please let us know and we will consider incorporating them in 
this file, as well.

SUMMARY:

Make sure you have the latest Windows SDK (see msdn.microsoft.com for more information) 
or this file may not compile!

This file contains a demo application that illustrates the use of the functions 
specified in the VistaTools.cxx file.

It uses TaskDialog for its user interface, that gives the user two choices how to restart itself:
as an elevated process, or as a limited (non-elevated) one. This way, you can see in action how a 
non-elevated process starts an elevated one, and vica versa. 

MODIFICATIONS:
	2006-Dec-16: created by Andrei Belogortseff.
	2007-Feb-20: updated to use RunNonElevated() instead of RunAsStdUser().
*/

#define NTDDI_VERSION NTDDI_LONGHORN

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers

#include <windows.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>
#include <commctrl.h>
#include <shellapi.h>

#include <strsafe.h>

//#define IMPLEMENT_VISTA_TOOLS - it's in the DLL now
#include "DLL\VistaTools.cxx"

#include "resource.h"

#pragma comment(lib, "comctl32.lib")

/// the text to be displayed as title of the messages:
LPCTSTR pszMsgTitle = 

#ifdef WIN64
	_T("Vista Elevator 2.0 (64-bit)");
#else
	_T("Vista Elevator 2.0 (32-bit)");
#endif

// this is the workhorce of this application, that uses the functions from VistaTools.cxx 
// to restarts itself elevated or not:
BOOL RunMyself( HWND hwnd, BOOL bElevated )
{
	TCHAR szPath[ MAX_PATH ];

	::GetModuleFileName( NULL, szPath, _tsizeof(szPath) );

	BOOL bSuccess = (bElevated ? RunElevated( hwnd, szPath ) : RunNonElevated( hwnd, szPath ) );

	if ( !bSuccess )
	{
		::MessageBox( NULL, 
			_T("Could not perform this action."),
			pszMsgTitle,
			MB_OK | MB_ICONSTOP );
	}

	return bSuccess;
}

/// the command IDs of the radio buttons on the Task Dialog:

#define ID_CMD_ELEVATE			2001
#define ID_CMD_LIMIT			2002

// need this to be global, because we need to set/access it from TaskDialogCallbackProc:
int	nRadioButton = 0;

HRESULT CALLBACK
TaskDialogCallbackProc(          
	__in HWND hwnd, 
	__in UINT msg, 
	__in WPARAM wParam, 
	__in LPARAM lParam, 
	__in LONG_PTR lpRefData
)
{
	switch( msg )
	{
	case TDN_RADIO_BUTTON_CLICKED:

		nRadioButton = (int)wParam;
		
		break;

	case TDN_BUTTON_CLICKED:
		
		if ( wParam == IDOK )
		{
			BOOL bElevate = (nRadioButton == ID_CMD_ELEVATE );

			RunMyself( hwnd, bElevate );
		}
		
		break;

	case TDN_HYPERLINK_CLICKED:
		
		ShellExecute( hwnd, NULL, (LPCTSTR)lParam, NULL, NULL, SW_SHOWNORMAL );
		
		break;
	}

	return S_OK;
}

/// this sets up the TaskDialog to display to the user:

HRESULT DlgMain(
	__in HWND		hwndParent,
	__in HINSTANCE	hInstance,
    __out_opt int *	pnButton,
    __out_opt int *	pnRadioButton )
{
	ASSERT( pnButton );
	ASSERT( pnRadioButton );

	TOKEN_ELEVATION_TYPE tet = TokenElevationTypeDefault;

	if ( FAILED( ::GetElevationType( &tet ) ) )
	{
		ASSERT( FALSE );
		return 0;
	}

	BOOL bElevated = FALSE;

	if ( FAILED( ::IsElevated( &bElevated ) ) )
	{
		ASSERT( FALSE );
		return 0;
	}

	*pnRadioButton = bElevated ? ID_CMD_ELEVATE : ID_CMD_LIMIT;

	LPCTSTR pszBtn1, pszBtn2;

	switch( *pnRadioButton ) // are we elevated now?
	{
	case ID_CMD_ELEVATE:
		pszBtn1 = _T("Remain elevated\nRestart as an elevated process.");
		pszBtn2 = _T("Limit\nRestart as a non-elevated process.");
		break;

	case ID_CMD_LIMIT:
		pszBtn1 = _T("Elevate\nRestart as an elevated process.");
		pszBtn2 = _T("Remain limited\nRestart as a non-elevated process.");
		break;
	default:
		return ERROR_INVALID_PARAMETER;
	}
    
    const TASKDIALOG_BUTTON buttons[] =
    {
		{ ID_CMD_ELEVATE,	pszBtn1 } ,
        { ID_CMD_LIMIT,		pszBtn2 } 
    };
       
    TASKDIALOGCONFIG tdConfig;

	memset( &tdConfig, 0, sizeof( tdConfig ) );
	
	tdConfig.cbSize					= sizeof(tdConfig);

    tdConfig.hwndParent              = hwndParent;
    tdConfig.hInstance               = hInstance;
    tdConfig.dwFlags                 = 
		TDF_ALLOW_DIALOG_CANCELLATION	| 
		TDF_EXPAND_FOOTER_AREA			|
		TDF_ENABLE_HYPERLINKS ;
    
    tdConfig.cRadioButtons			= ARRAYSIZE(buttons);
    tdConfig.pRadioButtons			= buttons;
    tdConfig.nDefaultRadioButton	= *pnRadioButton;

    tdConfig.pszWindowTitle          = pszMsgTitle;

    tdConfig.pszMainInstruction      = _T("VistaElevator 2.0");
    
	tdConfig.pszMainIcon             = bElevated ? TD_SHIELD_ICON : TD_INFORMATION_ICON;

	TCHAR szContent[ 256 ]; // should be long enough

	StringCchPrintf( szContent, _tsizeof(szContent), 
		_T("Current process:\n%s %s"),
		(tet == TokenElevationTypeDefault ? 
			_T("TokenElevationTypeDefault") : 
				(tet == TokenElevationTypeFull ? 
					_T("TokenElevationTypeFull") : 
					_T("TokenElevationTypeLimited") ) ),
		(bElevated ? _T("ELEVATED") : _T("NON-ELEVATED") ) );

	tdConfig.pszContent              = szContent;

    tdConfig.pszExpandedInformation  = _T("For more information about this tool, ") 
		_T("visit:<a href=\"http://www.tweak-uac.com/\">http://tweak-uac.com</a>");

    tdConfig.pszCollapsedControlText = _T("More information");
    tdConfig.pszExpandedControlText  = _T("Less information");

	tdConfig.pfCallback					= TaskDialogCallbackProc;

    HRESULT hr = TaskDialogIndirect( &tdConfig, pnButton, pnRadioButton, NULL );

    return hr;
}

/////////////////////////////////
// the main entry point

int APIENTRY 
_tWinMain(	
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPTSTR    lpCmdLine,
	int       nCmdShow
	)
{

	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

	if ( !IsVista() )
	{
		::MessageBox( NULL, 
			_T("This software is designed for Windows(R) Vista(TM) only.")
			_T("\n\nPlease visit http://www.tweak-uac.com for more information."),
			pszMsgTitle,
			MB_OK | MB_ICONSTOP );
		return 0;
	}

	#ifndef WIN64

	////////////////////////////////////////////////////////////////////
	// If 32-bit code is running under 64-bit Windows, we need to exit.
	// Need to compile x64 code to use it with a x64 verion of Windows.

	if ( IsWow64() ) 
	{
		::MessageBox( NULL, 
			_T("This software is designed for the 32-bit version of Windows(R) Vista(TM).")
			_T("\n\nPlease visit http://www.tweak-uac.com to download a 64-bit version of this utility."),
			pszMsgTitle,
			MB_OK | MB_ICONSTOP );
		return 0;
	}

	#endif//WIN64


	::InitCommonControls();

	int	nButton = 0;

	HRESULT hr = DlgMain( NULL,
					hInstance,
					&nButton,
					&nRadioButton );

	if ( FAILED( hr ) )
	{
		::MessageBox( NULL, 
			_T("Could not display the main dialg box."),
			pszMsgTitle,
			MB_OK | MB_ICONSTOP );
	}

	return (int) 0;
}
